// Nathan E. Moore
// March 23, 2004

#include "menusub.h"
#include <sstream>
#include <crtdbg.h>

bool MenuSub::s_autocolumn = false;
bool MenuSub::s_header = false;
long MenuSub::s_animation = TPM_NOANIMATION;
long MenuSub::s_position = TPM_LEFTALIGN | TPM_TOPALIGN;

// All the _ASSERT(-1 != location)'s are to prevent appending stuff to the end of the menu

void HmenuWrapper::insertEntry(const MenuEntryPtr& item, unsigned int location, bool column)
{
	_ASSERT( -1 != location);
	m_handle.InsertMenu(location, MF_BYPOSITION | MF_ENABLED | MF_STRING | (column ? MF_MENUBARBREAK : 0),
	                    item->getId(), item->getEntryText().c_str());
}

void HmenuWrapper::insertSubmenu(const MenuSubPtr& sub, unsigned int location, bool column)
{
	_ASSERT( -1 != location);
	m_handle.InsertMenu(location, MF_BYPOSITION | MF_ENABLED | MF_POPUP | MF_STRING | (column ? MF_MENUBARBREAK : 0),
	                    reinterpret_cast<UINT_PTR>(sub->m_menu.m_hMenu), sub->getEntryText().c_str());
	sub->m_parent.Attach(m_handle.m_hMenu);
}

void HmenuWrapper::insertSeparator(const unsigned int location, bool column)
{
	_ASSERT( -1 != location);
	m_handle.InsertMenu(location, MF_BYPOSITION | MF_SEPARATOR | (column ? MF_MENUBARBREAK : 0), MenuSub::NULL_ID);
}

void HmenuWrapper::insertDeadEntry(const std::wstring& txt, unsigned int location, bool column)
{
	_ASSERT( -1 != location);
	m_handle.InsertMenu(location, MF_BYPOSITION | MF_ENABLED | MF_STRING | MF_DISABLED | (column ? MF_MENUBARBREAK : 0),
	                    MenuSub::NULL_ID, txt.c_str());
}

bool HmenuWrapper::deleteMenuElement(unsigned int location)
{
	return m_handle.DeleteMenu(location, MF_BYPOSITION) != 0;
}

void HmenuWrapper::setColumnState(unsigned int location, bool column)
{
	MENUITEMINFO mii;
	mii.cbSize = sizeof(mii);
	mii.fMask = MIIM_FTYPE;
	m_handle.GetMenuItemInfo(location, MF_BYPOSITION, &mii);
	if (column)
		mii.fType |= MFT_MENUBARBREAK;
	else
		mii.fType &= ~MFT_MENUBARBREAK;
	m_handle.SetMenuItemInfo(location, MF_BYPOSITION, &mii);
}

bool HmenuWrapper::getColumnState(unsigned int location)
{
	MENUITEMINFO mii;
	mii.cbSize = sizeof(mii);
	mii.fMask = MIIM_FTYPE;
	m_handle.GetMenuItemInfo(location, MF_BYPOSITION, &mii);
	return MF_MENUBARBREAK == (mii.fType & MF_MENUBARBREAK);
}

void MenuSub::beginDisplay(int x, int y, HWND hwnd)
{
	if (!m_menu.TrackPopupMenuEx(TPM_LEFTBUTTON | s_animation | s_position, x, y, hwnd))
	{
		std::ostringstream oss;
		oss << "TrackPopupMenuEx Failed. " << WinErrorMsg(GetLastError()) << " " LOCATION;
		throw std::runtime_error(oss.str());
	}
}

void MenuSub::autocolumn(HWND hwnd)
// This code is based on the tallmenu SDK sample
{
	if (!(m_autocolumn && s_autocolumn)) // early out
		return ;

	int itemcount = m_menu.GetMenuItemCount();

	// use parent menu as it will have been shown before this menu, since GetMenuItemRect
	// only works on a menu that has been displayd
	int itemheight = 0;
	if (m_parent.IsMenu())
	{
		RECT itemsize;
		m_parent.GetMenuItemRect(hwnd, 0, &itemsize);
		itemheight = itemsize.bottom - itemsize.top;
	}
	// not an else if to protect against it haveing a parentmenu but that menu not being shown befor this one
	// not sure how that would happen but one 'if' == nothing
	if (itemheight == 0)
		itemheight = GetSystemMetrics(SM_CYMENU);

	POINT pt;
	GetCursorPos(&pt);
	RECT area;
	getMonitorArea(&pt, &area);
	int screenheight = area.bottom - area.top - 3 * itemheight;

	// TODO: at the moment this uses a static array, it should eventually check the return value of
	// GetMenuItemInfo are resize the buffer if needed.
	MENUITEMINFO mii;
	wchar_t buf[512];

	for (int i = 0, currentheight = 0; i < itemcount; i++)
	{
		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_TYPE | MIIM_DATA;
		mii.dwTypeData = buf;
		mii.cch = sizeof(buf) / sizeof(buf[0]);
		m_menu.GetMenuItemInfo(i, MF_BYPOSITION, &mii);

		if (currentheight > screenheight)
		{
			currentheight = 0;
			mii.fType |= MFT_MENUBARBREAK;
		}
		else
			mii.fType &= ~MFT_MENUBARBREAK;

		m_menu.SetMenuItemInfo(i, MF_BYPOSITION, &mii);
		currentheight += itemheight;
	}
}

void MenuSub::setAnimation(const char* horzmode, const char* vertmode)
{
	long horz = 0L;
	long vert = 0L;
	if (lstrcmpiA(horzmode, "off") == 0)
		horz = TPM_NOANIMATION;
	else if (lstrcmpiA(horzmode, "right") == 0)
		horz = TPM_HORNEGANIMATION;
	else if (lstrcmpiA(horzmode, "left") == 0)
		horz = TPM_HORPOSANIMATION;

	if (lstrcmpiA(vertmode, "off") == 0)
		vert = TPM_NOANIMATION;
	else if (lstrcmpiA(vertmode, "up") == 0)
		vert = TPM_VERNEGANIMATION;
	else if (lstrcmpiA(vertmode, "down") == 0)
		vert = TPM_VERPOSANIMATION;

	if (horz == TPM_NOANIMATION && vert == TPM_NOANIMATION)
		s_animation = TPM_NOANIMATION;
	else if (horz == TPM_NOANIMATION)
		s_animation = vert;
	else if (vert == TPM_NOANIMATION)
		s_animation = horz;
	else
		s_animation = horz | vert;
}

void MenuSub::setPosition(const char* horzmode, const char* vertmode)
{
	if (lstrcmpiA(horzmode, "left") == 0)
		s_position = TPM_LEFTALIGN;
	else if (lstrcmpiA(horzmode, "center") == 0)
		s_position = TPM_CENTERALIGN;
	else if (lstrcmpiA(horzmode, "right") == 0)
		s_position = TPM_RIGHTALIGN;

	if (lstrcmpiA(vertmode, "top") == 0)
		s_position |= TPM_TOPALIGN;
	else if (lstrcmpiA(vertmode, "center") == 0)
		s_position |= TPM_VCENTERALIGN;
	else if (lstrcmpiA(vertmode, "bottom") == 0)
		s_position |= TPM_BOTTOMALIGN;
}
