// Nathan E. Moore
// March 30, 2004

#include "dateparser.h"
#include <sstream>
#include <ctime>
#include <boost/scoped_array.hpp>

namespace date_impl
{
	const std::wstring DateEntry::S_INET_TIME_PATTERN(L"%@");
	const size_t DateEntry::S_FORMATED_BUFFER_SIZE = 128;

	void DateEntry::update()
	{
		__time64_t tsec;
		_time64(&tsec);
		std::wstring rval(formatTime(m_pattern, _localtime64(&tsec)));

		size_t location = rval.find(S_INET_TIME_PATTERN);
		if (std::wstring::npos != location)
		{
			// remove the %@
			rval.erase(location, S_INET_TIME_PATTERN.length());
			std::wostringstream woss;
			woss << getInetTime(tsec);
			rval.insert(location, woss.str());
		}

		setEntryText(rval);
	}

	void DateEntry::execute(HWND hwnd)
	{
		// Taken from popup2, and converted to unicode
		// Taken from aLSClock
		wchar_t winDir[MAX_PATH + 1];
		GetSystemDirectory(winDir, MAX_PATH);
		ShellExecute(hwnd, L"open", L"rundll32.exe", L"shell32.dll,Control_RunDLL timedate.cpl", winDir, SW_SHOWNORMAL);
	}

	// ATL has a CTime class that will do this for you, however it uses CString and I don't
	std::wstring DateEntry::formatTime(const std::wstring& pattern, tm *timedata)
	{
		wchar_t sbuf[S_FORMATED_BUFFER_SIZE];
		if (wcsftime(sbuf, S_FORMATED_BUFFER_SIZE, pattern.c_str(), timedata))
			return sbuf;
		else	// use a larger dynamic buffer
		{
			// I think that it is highly unlikly that this code will ever be called
			// as popup2 only supports 128 chars, and any more than that is
			// incredibly long for a menu
			size_t size = pattern.length() + S_FORMATED_BUFFER_SIZE;
			boost::scoped_array<wchar_t> dbuf(new wchar_t[size]);
			wcsftime(dbuf.get(), size, pattern.c_str(), timedata);
			return std::wstring(dbuf.get());
		}
	}

	// stolen from popup2 (re5ources.cpp)
	// modified for __time64_t
	int DateEntry::getInetTime(__time64_t now)
	{
		// and one hour to get GMT+1 (swizz based time :)
		__time64_t nDawn = now + 3600;

		// get number of seconds since midnight
		__time64_t nSeconds = nDawn % 86400;

		// convert the number of seconds to a number 0..1000
		int nInetTime = (int)(nSeconds / 86.4);

		// account for possible overlapp
		if (nInetTime >= 1000)
			nInetTime = 0;
		return nInetTime;
	}
}