// Nathan E. Moore
// March 22, 2004

#pragma once
#include "parser.h"
#include "menusub.h"
#include "globals.h"
#include <vector>
#include <list>
#include <string>
#include <boost/mem_fn.hpp>

class ManualParser;
typedef boost::shared_ptr<ManualParser> ManualParserPtr;

class ManualParser : public StaticParserBase
{
private:
	typedef std::vector<MenuEntryPtr> EntryVector;
	typedef std::vector<MenuSubPtr> SubVector;
	typedef std::pair<unsigned char, unsigned char> PatternElement;
	// these are lists so there storage is actually freed on a clear()
	typedef std::list<std::wstring> DeadEntryList;
	typedef std::list<PatternElement> PatternList;

	// I should consider make these a enum is the saving of 3 bytes per Element
	// really worth it?
	// constant types used with Pattern
	static const unsigned char S_ENTRY = 0;
	static const unsigned char S_SUBMENU = 1;
	static const unsigned char S_SEPARATOR = 2;
	static const unsigned char S_DEAD_ENTRY = 3;
	// constant flags used with Pattern
	static const unsigned char S_NEW_COLUMN = 1;

	EntryVector m_entries;
	SubVector m_subs;
	DeadEntryList m_deadentries;
	PatternList m_pattern;

	ManualParser(const std::wstring& txt)
		:StaticParserBase(txt)
	{}

protected:
	// ParserBase
	unsigned int parse(HmenuInterface& menu, unsigned int start);

	void cleanupChildren()
	{
		std::for_each(m_subs.begin(), m_subs.end(), boost::mem_fn(&MenuSub::endDisplay));
	}

public:
	//ParserInterface (part not covered by ParserBase)
	MenuEntryPtr findEntry(unsigned int id) const;
	MenuSubPtr findSubmenu(HMENU hmenu) const;

	bool doContextMenu(HwndInterface&)
	{
		return false;
	}

	void appendEntry(const MenuEntryPtr& item, bool column = false)
	{
		m_entries.push_back(item);
		m_pattern.push_back(PatternElement(S_ENTRY, column ? S_NEW_COLUMN : 0));
	}

	void appendSubmenu(const MenuSubPtr& sub, bool column = false)
	{
		m_subs.push_back(sub);
		m_pattern.push_back(PatternElement(S_SUBMENU, column ? S_NEW_COLUMN : 0));
	}

	void appendSeparator(bool column = false)
	{
		m_pattern.push_back(PatternElement(S_SEPARATOR, column ? S_NEW_COLUMN : 0));
	}

	void appendDeadEntry(const std::wstring& txt, bool column = false)
	{
		m_deadentries.push_back(txt);
		m_pattern.push_back(PatternElement(S_DEAD_ENTRY, column ? S_NEW_COLUMN : 0));
	}

	static ManualParserPtr createManualParser(const std::wstring& txt)
	{
		ManualParserPtr temp(new ManualParser(txt));
		return temp;
	}
};
