// Nathan Moore
// April 11, 2004

#include "globals.h"
#include "menu.h"
#include "menusettings.h"

static Menu* g_menu;

extern "C"
{
	// Litestep module interface
	// no prototype
	__declspec( dllexport ) int initModuleEx(HWND parentwnd, HINSTANCE, LPCSTR)
	{
		CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
		// a try block here because litestep doesn't handle exceptions
		try
		{
			g_menu = new Menu(parentwnd);
		}
		catch (std::exception& e)
		{
			LSLOG(lslog::error << "Initialization Failed. " << e.what() << " " LOCATION);
			return 1;
		}
		catch (...)
		{
			LSLOG(lslog::error << "Intialization Failed. A unknown exeception was raised. " LOCATION);
			return 1;
		}
		return 0;
	}

	// no prototype
	__declspec( dllexport ) void quitModule(HINSTANCE)
	{
		delete g_menu;
		CoUninitialize();
	}

	// Bang command routing
	void onBangCmdFunction(HWND, const char* bang, const char* args)
	{
		// hide any exceptions from litestep
		try
		{
			if (g_menu)
				g_menu->onBangCmd(bang, args);
		}
		catch (std::exception& e)
		{
			LSLOG(lslog::error << "Bang \"" << bang << "\" Failed. " << e.what() << " " LOCATION);
		}
		catch (...)
		{
			LSLOG(lslog::error << "Bang \"" << bang << "\" failed. An unknown exception was raised. " LOCATION);
		}
	}
}

Menu::Menu(HWND parentwnd)
		: m_parenthwnd(parentwnd)
{
	Create(m_parenthwnd);
	MenuSettings(*this);

	UINT msgs[] = {LM_POPUP, LM_HIDEPOPUP, LM_GETREVID, 0};
	::SendMessageA(GetLitestepWnd(), LM_REGISTERMESSAGE, reinterpret_cast<WPARAM>(m_hWnd), reinterpret_cast<LPARAM>(msgs));
}

Menu::~Menu()
{
	UINT msgs[] = {LM_POPUP, LM_HIDEPOPUP, LM_GETREVID, 0};
	::SendMessageA(GetLitestepWnd(), LM_UNREGISTERMESSAGE, reinterpret_cast<WPARAM>(m_hWnd), reinterpret_cast<LPARAM>(msgs));

	for (BangMap::const_iterator i = m_bangs.begin(), end = m_bangs.end(); i != end; ++i)
	{
		// Destroy other windows
		(*i).second->DestroyWindow();
		// Unregister bang commands
		RemoveBangCommand((*i).first.c_str());
	}

	DestroyWindow();
}

void Menu::addBang(const std::string& cmdstr, const MenuSubPtr& popup)
{
	BangableMenuWindowPtr popupwnd(new BangableMenuWindow(popup));
	popupwnd->Create(m_parenthwnd);
	if (AddBangCommandEx(cmdstr.c_str(), &onBangCmdFunction))
	{
		std::pair<BangMap::iterator, bool> insertres = m_bangs.insert(std::make_pair(cmdstr, popupwnd));
		if (!insertres.second)
		{
			popupwnd->DestroyWindow();
			RemoveBangCommand(cmdstr.c_str());
			LSLOG(lslog::error << "Duplicate menu bangs found: " << cmdstr);
		}
	}
	else
		LSLOG(lslog::error << "Adding bang " << cmdstr << " failed.");
}
