// Nathan E. Moore
// March 23, 2004

#include "shellparser.h"
#include "menusub.h"
#include <algorithm>

MenuEntryPtr ShellParser::findEntry(unsigned int id) const
{
	// in here
	MenuIdListSeq::const_iterator iter = std::find_if(m_midlsts.begin(), m_midlsts.end(), EntrySearcher(id));
	if (iter != m_midlsts.end())
		return boost::static_pointer_cast<MenuEntryPtr::element_type>(*iter);

	// in a submenu
	for (SubList::const_iterator i = m_subs.begin(), end = m_subs.end(); i != end; ++i)
	{
		MenuEntryPtr item = (*i)->getEntry(id);
		if (item)
			return item;
	}

	// not in this menu hierarchy
	return MenuEntryPtr();
}

MenuSubPtr ShellParser::findSubmenu(HMENU hmenu) const
{
	DEBUG_WOUT(L"ShellParser::findSubmenu: " << getEntryText());
	// in here
	SubList::const_iterator iter = std::find_if(m_subs.begin(), m_subs.end(), SubSearcher(hmenu));
	if (iter != m_subs.end())
		return *iter;

	// in a submenu
	for (SubList::const_iterator i = m_subs.begin(), end = m_subs.end(); i != end; ++i)
	{
		MenuSubPtr sub = (*i)->getSub(hmenu);
		if (sub)
			return sub;
	}

	// not in this menu hierarchy
	return MenuSubPtr();
}

unsigned int ShellParser::parse(HmenuInterface& menu, unsigned int start)
{
	typedef std::vector<ShellParserPtr> ShellParserVector;
	ShellParserVector sparsers;

	// fill sparsers and m_midlsts
	CComPtr<IShellFolder> folder;
	if (m_idlist->mkid.cb == 0)	// Allow !PopupDesktop to work
		folder = util::getDesktop();
	else if (FAILED(util::getDesktop()->BindToObject(m_idlist, NULL, IID_IShellFolder, reinterpret_cast<void**>(&(folder.p)))))
		throw std::runtime_error(ERR_MSG("BindToObject Failed"));

	CComPtr<IEnumIDList> enumerator;
	if (FAILED(folder->EnumObjects(NULL, SHCONTF_NONFOLDERS | SHCONTF_FOLDERS , &(enumerator.p))))
		throw std::runtime_error(ERR_MSG("Could not get IEnumIDList"));

	ITEMIDLIST* pidl;
	HRESULT hres;
	util::ItemIdList currentitem;
	while (SUCCEEDED(hres = enumerator->Next(1, &pidl, NULL)) && hres != S_FALSE )
	{
		currentitem.Free();
		currentitem.Attach(pidl);
		STRRET name;
		// Find Display Name
		if (SUCCEEDED(folder->GetDisplayNameOf(currentitem, SHGDN_NORMAL, &name)))
		{
			wchar_t* strname;
			// Just Skip the item if we can't get a display name
			if (SUCCEEDED(StrRetToStr(&name, currentitem, &strname)))
			{
				try
				{
					DWORD attrib = SFGAO_FOLDER | SFGAO_REMOVABLE;
					if (SUCCEEDED(folder->GetAttributesOf(1, const_cast<const ITEMIDLIST**>(&currentitem.m_pData), &attrib)))
					{
						util::ItemIdList newidlist = m_idlist.concat(currentitem);
						if ((attrib & SFGAO_FOLDER) == SFGAO_FOLDER && (attrib & SFGAO_REMOVABLE) != SFGAO_REMOVABLE)
							sparsers.push_back(createShellParser(strname, newidlist));
						else
							m_midlsts.push_back(shell_impl::MenuIdList::createMenuIdList(strname, newidlist));
					}
					CoTaskMemFree(strname);
				}
				catch (std::exception& e)
				{
					LSLOG(lslog::debug << e.what());
					CoTaskMemFree(strname);
				}
			}
		}
	}
	// end fill

	std::sort(sparsers.begin(), sparsers.end(), shell_impl::Sorter<ShellParserPtr>());
	m_midlsts.sort(shell_impl::Sorter<MenuIdListPtr>());

	// This loop will merge the two vectors
	ShellParserVector::iterator f = sparsers.begin();
	MenuIdListSeq::iterator i = m_midlsts.begin();
	shell_impl::Sorter<ShellParserPtr, MenuIdListPtr> comp;
	unsigned int location = start;
	while (f != sparsers.end() && i != m_midlsts.end())
	{
		if (comp(*f, *i))
		{
			MenuSubPtr msptr(MenuSub::createMenuSub(*f, true));
			menu.insertSubmenu(msptr, location);
			m_subs.push_front(msptr);
			++f;
		}
		else
		{
			menu.insertEntry(boost::static_pointer_cast<MenuEntryPtr::element_type>(*i), location);
			++i;
		}
		++location;
	}

	if (f != sparsers.end())
	{
		for ( ; f != sparsers.end(); ++f, ++location)
		{
			MenuSubPtr msptr(MenuSub::createMenuSub(*f, true));
			menu.insertSubmenu(msptr, location);
			m_subs.push_front(msptr);
		}
	}
	else if (i != m_midlsts.end())
	{
		for ( ; i != m_midlsts.end(); ++i, ++location)
			menu.insertEntry(boost::static_pointer_cast<MenuEntryPtr::element_type>(*i), location);
	}
	return location - start;
}
