// Nathan E. Moore
// March 23, 2004

#pragma once
#include "parser.h"
#include "idlist.h"
#include "menuentry.h"
#include <list>
#include <slist>
#include <atlcomcli.h>

class ShellParser;

namespace shell_impl
{
	// this is a template because shared_ptrs don't allow for implicit upcasts
	// but if they did this would work on ShellBase objs
	template <class lhs, class rhs = lhs>
	class Sorter : public std::binary_function<lhs, rhs, bool>
	{
	private:
		CComPtr<IShellFolder> desk;
	public:
		Sorter()
			:desk(util::getDesktop())
		{}

		bool operator()(const lhs& left, const rhs& right) const
		{
			HRESULT hres = desk->CompareIDs(0, left->m_idlist, right->m_idlist);
			return static_cast<short>(HRESULT_CODE(hres)) < 0;
		}
	};

	class MenuIdList;

	class ShellBase
	{
	protected:
		typedef boost::shared_ptr<MenuIdList> MenuIdListPtr;

		util::ItemIdList m_idlist;

		explicit ShellBase(util::ItemIdList& idlist)
			: m_idlist(idlist)
		{}

		explicit ShellBase(int csidl)
			: m_idlist(util::createVirtualFolder(csidl))
		{}

	public:
		virtual ~ShellBase()
		{}

		friend class Sorter;
	};

	class MenuIdList :
		public MenuEntryBase,
		public ShellBase
	{
	private:
		MenuIdList(const std::wstring& txt, util::ItemIdList& idlist)
			:MenuEntryBase(txt),
			ShellBase(idlist)
		{}

		MenuIdList(const std::wstring& txt, int csidl)
			:ShellBase(csidl),
			MenuEntryBase(txt)
		{}

		MenuIdList(int csidl)
			:ShellBase(csidl),
			MenuEntryBase()
		{
			setEntryText(m_idlist.getDisplayName());
		}

		static MenuIdListPtr createMenuIdList(const std::wstring& txt, util::ItemIdList& idlist)
		{
			MenuIdListPtr temp(new MenuIdList(txt, idlist));
			return temp;
		}

	public:
		bool displayContextMenu(HwndInterface& hi)
		{
			return util::popupContextMenu(m_idlist, hi);
		}

		void execute(HWND hwnd)
		{
			m_idlist.execute(hwnd);
		}

		friend class ShellParser;

		static MenuEntryPtr createMenuIdList(const std::wstring& txt, int csidl)
		{
			MenuEntryPtr temp(new MenuIdList(txt, csidl));
			return temp;
		}

		static MenuEntryPtr createMenuIdList(int csidl)
		{
			MenuEntryPtr temp(new MenuIdList(csidl));
			return temp;
		}
	};
}

class ShellParser :
	public DynamicParserBase,
	public shell_impl::ShellBase
{
private:
	typedef std::list<MenuIdListPtr> MenuIdListSeq;
	typedef std::slist<MenuSubPtr> SubList;
	typedef boost::shared_ptr<ShellParser> ShellParserPtr;

	MenuIdListSeq m_midlsts;
	SubList m_subs;

	ShellParser(int csidl)
		:ShellBase(csidl),
		DynamicParserBase()
	{
		setEntryText(m_idlist.getDisplayName());
	}

	ShellParser(const std::wstring& txt, int csidl)
		:ShellBase(csidl),
		DynamicParserBase(txt)
	{}

	ShellParser(const std::wstring& txt, util::ItemIdList& idlist)
		:ShellBase(idlist),
		DynamicParserBase(txt)
	{}

	static ShellParserPtr createShellParser(const std::wstring& txt, util::ItemIdList& idlist)
	{
		ShellParserPtr temp(new ShellParser(txt, idlist));
		return temp;
	}

protected:
	unsigned int parse(HmenuInterface& menusub, unsigned int start);

	void clearStorage()
	{
		m_midlsts.clear();
		m_subs.clear();
	}

public:
	MenuEntryPtr findEntry(unsigned int id) const;
	MenuSubPtr findSubmenu(HMENU hmenu) const;

	bool doContextMenu(HwndInterface& hi)
	{
		return util::popupContextMenu(m_idlist, hi);
	}

	static ParserPtr createShellParser(int csidl)
	{
		ParserPtr temp(new ShellParser(csidl));
		return temp;
	}

	static ParserPtr createShellParser(const std::wstring& txt, int csidl)
	{
		ParserPtr temp(new ShellParser(txt, csidl));
		return temp;
	}
};