// Nathan E. Moore
// March 23, 2004

#pragma once
#include "menuentry.h"
#include "globals.h"
#include "parser.h"
#include <slist>
#include <string>

class TasksParser : public DynamicParserBase
{
private:
	typedef struct
	{
		TasksParser* parser;
		HmenuInterface* menu;
		unsigned int location;
	} TasksParseData;
	
	typedef std::slist<MenuEntryPtr> EntryList;
	EntryList m_wnds;

	// prevent creation of a a TasksParser on the stack
	TasksParser(const std::wstring& txt = G_TASKS)
		:DynamicParserBase(txt)
	{}

	static BOOL WINAPI enumWindowsProc(HWND hwnd, LPARAM lparam);

protected:
	unsigned int parse(HmenuInterface& menu, unsigned int start)
	{
		TasksParseData pdata;
		pdata.parser = this;
		pdata.menu = &menu;
		pdata.location = start;

		if (!EnumWindows(enumWindowsProc, reinterpret_cast<LPARAM>(&pdata)))
		{
			std::ostringstream oss;
			oss << "EnumWindows Failed. " << WinErrorMsg(GetLastError()) << " " LOCATION;
			throw std::runtime_error(oss.str());
		}
		return pdata.location - start;
	}

	void clearStorage()
	{
		m_wnds.clear();
	}

public:
	MenuEntryPtr findEntry(unsigned int id) const
	{
		EntryList::const_iterator i = std::find_if(m_wnds.begin(), m_wnds.end(), EntrySearcher(id));
		if (m_wnds.end() == i)
			return MenuEntryPtr();
		else
			return *i;
	}

	MenuSubPtr findSubmenu(HMENU) const
	{
		return MenuSubPtr();
	}

	bool doContextMenu(HwndInterface&)
	{
		return false;
	}

	static ParserPtr createTasksParser(const std::wstring& txt = G_TASKS)
	{
		ParserPtr temp(new TasksParser(txt));
		return temp;
	}
};
